"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MultisigSign = void 0;
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/. */
const sdk_1 = require("@ironfish/sdk");
const core_1 = require("@oclif/core");
const command_1 = require("../../../../command");
const flags_1 = require("../../../../flags");
const longPrompt_1 = require("../../../../utils/longPrompt");
class MultisigSign extends command_1.IronfishCommand {
    async start() {
        const { flags } = await this.parse(MultisigSign);
        const signingPackage = flags.signingPackage?.trim() ??
            (await (0, longPrompt_1.longPrompt)('Enter the signing package', { required: true }));
        let signatureShares = flags.signatureShare;
        if (!signatureShares) {
            const input = await (0, longPrompt_1.longPrompt)('Enter the signature shares separated by commas', {
                required: true,
            });
            signatureShares = input.split(',');
        }
        signatureShares = signatureShares.map((s) => s.trim());
        core_1.CliUx.ux.action.start('Signing the multisig transaction');
        const client = await this.sdk.connectRpc();
        const response = await client.wallet.multisig.aggregateSignatureShares({
            account: flags.account,
            broadcast: flags.broadcast,
            signingPackage,
            signatureShares,
        });
        const bytes = Buffer.from(response.content.transaction, 'hex');
        const transaction = new sdk_1.Transaction(bytes);
        core_1.CliUx.ux.action.stop();
        if (flags.broadcast && response.content.accepted === false) {
            this.warn(`Transaction '${transaction.hash().toString('hex')}' was not accepted into the mempool`);
        }
        if (flags.broadcast && response.content.broadcasted === false) {
            this.warn(`Transaction '${transaction.hash().toString('hex')}' failed to broadcast`);
        }
        this.log(`Transaction: ${response.content.transaction}`);
        this.log(`Hash: ${transaction.hash().toString('hex')}`);
        this.log(`Fee: ${sdk_1.CurrencyUtils.renderIron(transaction.fee(), true)}`);
    }
}
MultisigSign.description = 'Aggregate signature shares from participants to sign a transaction';
MultisigSign.hidden = true;
MultisigSign.flags = {
    ...flags_1.RemoteFlags,
    account: core_1.Flags.string({
        char: 'f',
        description: 'Account to use when aggregating signature shares',
        required: false,
    }),
    signingPackage: core_1.Flags.string({
        char: 'p',
        description: 'Signing package',
    }),
    signatureShare: core_1.Flags.string({
        char: 's',
        description: 'Participant signature share',
        multiple: true,
    }),
    broadcast: core_1.Flags.boolean({
        default: true,
        allowNo: true,
        description: 'Broadcast the transaction to the network after signing',
    }),
};
exports.MultisigSign = MultisigSign;
//# sourceMappingURL=aggregate.js.map